#ifndef _ORION_H
#define _ORION_H

#define ORION_IQIK_M_CAL_MAX	64
#define ORION_IQIK_M_CAL_MID	32

#include "standard.h"

typedef enum
{
    OrionMsg_OK,
    OrionMsg_ERROR = 0x11110001,
    OrionMsg_Param_not_match,
} OrionMsg;

typedef enum
{
	ORION_GAIN          = 0x0001,		/*return ORION_LNA_GAIN, ORION_PGC_GAIN, ORION_PGA1_GAIN, and ORION_PGA2_GAIN*/
	ORION_LNA_GAIN      = 0x0002,
	ORION_PGC_GAIN      = 0x0004,
	ORION_PGA1_GAIN     = 0x0008,
	ORION_PGA2_GAIN     = 0x0010,
	ORION_RF_BACKOFF    = 0x0020,
	ORION_BB_BACKOFF    = 0x0040,
	ORION_DCC           = 0x0080,		/*return ORION_DCC_I and ORION_DCC_Q*/
	ORION_DCC_I         = 0x0100,
	ORION_DCC_Q         = 0x0200,
	ORION_RF_GMAX       = 0x0400,
	ORION_TOTAL_GMAX    = 0x0800,
	ORION_P_INBAND_SHIFT= 0x1000,
	ORION_P_ALL         = 0x2000,	/*return ORION_P_RSSI, ORION_P_TOTAL, and ORION_P_INBAND*/
    ORION_PARAM_ALL     = 0xFFFF,
} OrionParam_enum;

typedef struct
{
    struct {
        int Gain1;
        int Gain2;
        int Gain3;
        int Gain4;
        int RF_Backoff;
        int BB_Backoff;
    } Gain;

    struct {
        int DCC1;
        int DCC2;
    } DCC;
    
    int RF_Gmax;
    int Total_Gmax;
    int P_Inband_Shift;

    struct {
        int P1;
        int P2;
        int P3;
        int P4;
        int RF_Backoff;
        int BB_Backoff;
        int RF_Gain;
        int Total_Gain;
    } P_All;

} OrionParam;

extern Demodulator* Afatech_ORION_demodulator;
extern Byte Afatech_ORION_chip;

unsigned long orion_init(void);
unsigned long orion_setfreq(unsigned int bw_kHz, unsigned int rf_freq_kHz);
unsigned long orion_rmr(unsigned short mem_addr, unsigned char len, unsigned char *buf);
unsigned long orion_wmr(unsigned short mem_addr, unsigned char len, unsigned char *val);

unsigned int orion_get_cal_freq_iqik(unsigned int rf_freq_kHz, unsigned char iqik_m_cal);
unsigned int orion_get_freq_code(unsigned int rf_freq_kHz, unsigned int *nc, unsigned int *nv, unsigned int *mv);
unsigned int orion_get_nc(unsigned int rf_freq_kHz);
unsigned int orion_get_nv(unsigned int nc);
unsigned char orion_get_lna_cap_sel(unsigned int rf_freq_kHz);
unsigned int orion_get_lo_freq(unsigned int rf_freq_kHz);
unsigned char orion_get_lpf_bw(unsigned int bw_kHz);
/*
unsigned long orion_get_param(ORION_Param param, int *buf);
unsigned long orion_set_param(ORION_Param param, int *val);
*/
unsigned long orion_get_param(OrionParam_enum items, OUT OrionParam *data);
unsigned long orion_set_param(OrionParam_enum items, IN OrionParam *data);
#endif