#include "it9135.h"

DEVICE_CONTEXT DC;
bool TunerInited0, TunerInited1, DeviceReboot;
struct semaphore my_sem;
DVB_DEFINE_MOD_OPT_ADAPTER_NR(adapter_nr);

static int it9135_probe(struct usb_interface *intf,
		const struct usb_device_id *id)
{
	int retval = -ENOMEM;
	int i;

	deb_data("===it9135 usb device pluged in!! ===\n");
	retval = Device_init(interface_to_usbdev(intf),&DC, true);
	if (retval){
                if(retval) deb_data("Device_init Fail: 0x%08x\n", retval);
		return retval;
        }

	//init mutex
	init_MUTEX (&my_sem);
	
	if ( it9135_device_count > 2)
		it9135_device_count = 2;
	
	for (i = 0; i < it9135_device_count; i++)
	{
#if LINUX_VERSION_CODE >  KERNEL_VERSION(2,6,18)
	    if (DC.architecture == Architecture_PIP)
		it9135_properties[i].num_adapters = 2;
#endif

#if LINUX_VERSION_CODE <=  KERNEL_VERSION(2,6,25)
	    if (dvb_usb_device_init(intf, &it9135_properties[i], THIS_MODULE, NULL) == 0)
		{deb_data("dvb_usb_device_init success!!\n");return 0;}
#else
	    if (dvb_usb_device_init(intf, &it9135_properties[i], THIS_MODULE, NULL,adapter_nr) == 0)
		{deb_data("dvb_usb_device_init success!!\n");return 0;}
#endif
	}

	return -ENOMEM;
}
static int it9135_suspend(struct usb_interface *intf, u32 state)
{
	int error;
	deb_data("Enter %s Function\n",__FUNCTION__);

#ifdef EEEPC
	error = DL_Reboot();
#else
    if (DevicePower)
    {
	error = DL_CheckTunerInited(0, &TunerInited0);
	error = DL_CheckTunerInited(1, &TunerInited1);
	
	//Omega used, need this sequence.
	DL_TunerSuspend(PDC, 0, false);
	error = DL_ApCtrl(0, 0);
	//Omega used, need this sequence.
	DL_TunerSuspend(PDC, 1, false);
	error = DL_ApCtrl(1, 0);
	if (error) {deb_data("DL_ApCtrl error : 0x%x\n", error);}

	DeviceReboot = true;
    }
#endif
	return 0;
}

static int it9135_resume(struct usb_interface *intf)
{
	int retval = -ENOMEM;
	int error;
	deb_data("Enter %s Function\n",__FUNCTION__);

#ifdef EEEPC
#else
    if(DeviceReboot == true)
    {
	retval = Device_init(interface_to_usbdev(intf),&DC, false);
        if (retval)
                if(retval) deb_data("Device_init Fail: 0x%08x\n", retval);

	if (TunerInited0){
	    error = DL_ApCtrl(0, 1);
	    //Omega used		
	    DL_TunerSuspend(PDC, 0, true);
	}
	if (TunerInited1){
	    error = DL_ApCtrl(1, 1);
	    //Omega used		
	    DL_TunerSuspend(PDC, 1, true);
	}
    }
#endif
        return 0;
}


static struct usb_driver it9135_driver = {
#if LINUX_VERSION_CODE <=  KERNEL_VERSION(2,6,15)
	.owner = THIS_MODULE,
#endif
	.name       = "dvb_usb_it9135",
	.probe      = it9135_probe,
	.disconnect = dvb_usb_device_exit,
	.id_table   = it9135_usb_id_table,
//	.suspend    = it9135_suspend,
//	.resume     = it9135_resume,
};

static int __init it9135_module_init(void)
{
	int result;

	info("%s",__FUNCTION__);
	deb_data("dvb_usb_it9135 Module is loaded \n");

	if ((result = usb_register(&it9135_driver))) {
		err("usb_register failed. Error number %d",result);
		return result;
	}
	return 0;
}

static void __exit it9135_module_exit(void)
{
	deb_data("dvb_usb_it9135 Module is unloaded!\n");
	usb_deregister(&it9135_driver);
}

module_init (it9135_module_init);
module_exit (it9135_module_exit);

MODULE_AUTHOR("Jimmy Chen <JyunYu.Chen@ite.com.tw>");
MODULE_DESCRIPTION("Driver for devices based on ITEtech IT9135");
MODULE_VERSION(DRIVER_RELEASE_VERSION);
MODULE_LICENSE("GPL");
